/*
  Mini Breadboard Prototyping
 This example shows how to use the basics of classic Debounce sketch
 to interact with a pushbutton and halt the executing loop.
 In applications such as datalogging it is desireable to have
 a more predictable/orderly shutdown than may be accomplished 
 by simply disconnecting from a power supply.
 There are better strategies to meet this goal including non-blocking code
 and hardware interupts. The goal here is not in the presentation of the
 method bt to illustrate the Mini Breadboard Prototyping pushbuttion.
 Holding the pushbutton down for the length time specfied will toggle the LED and 
 halt the loop() execution

 Circuit:
  Prototyping Shield Pushbutton to Arduino Pin 2
  Prototyping Shield LED to Arduino Pin 9

 
 Created  March 2016
 For Instructable
 By TrewRoad

 */
const uint8_t  e_buttonPin=2;
const uint8_t  e_ledPin=9;
const int e_DeLayBounce=1250; //this is milliseconds
int wishedhalt;               // stop process if toggled off
long fademsg=499;              // keep message under control
long cnt_loop=0;              // loop counter
void setup() {
  // Open serial communications and wait for port to open:
  Serial.begin(9600);
 
  Serial.println("Use a Pushbutton to a Terminate Process");
/* Any code required to initialize SD cards, pin modes
 *  sensor starting would be here.
 */
 // Set the parameters for the pushbutton  
 wishedhalt = bounce_setup(e_buttonPin,e_ledPin,e_DeLayBounce);
  Serial.print("Button setup ");
  if (wishedhalt) {
    Serial.println("HIGH");
  } else {
    Serial.println("LOW");
  }
}
// The sketch process loop
void loop() {
 //The pushbutton should be evaluated before any loop activity
 //The pushbutton delay time will occur before any process.
 //If 'halted' the condition are those at the end of the last loop. 
 if (wishedhalt != g_buttonState(e_buttonPin,e_ledPin) ) {
  // in case there is some activity needed to preserve data or  
  // set shutdown conditions. The code would be here.
  // example: dataFile.close();
  Serial.println("Asked for a Halt?\nFlashing the LED for Goodbye?");
  pindoubleblink(e_ledPin,300,150, 3);   // Flash the LED twice  will 
  pindoubleblink(e_ledPin,150,80, 6);
  pindoubleblink(e_ledPin,80,40, 12);
  pindoubleblink(e_ledPin,40,20, 24);
  Serial.println("Loop halted. Safe to Power off.");
  while(true);
 }
 //The main process of the sketch would be here
 if (cnt_loop > fademsg) {
    Serial.print("Loop processed ");
    Serial.print(cnt_loop);
    Serial.println(" Times\nHold Pushbutton to Halt");
    fademsg=fademsg/2 + fademsg;
 }
 cnt_loop++;

}

